package consul

import (
	"fmt"
	"log"
	"net/http"
	"strconv"
	"time"

	consul "github.com/hashicorp/consul/api"
)

type Service struct {
	AppID       string
	Name        string
	Domain      string
	Address     string
	Port        int
	TTL         time.Duration
	ConsulAgent *consul.Agent
}

var ErrServiceUnavailable = fmt.Errorf("Service is unavailable")

func NewService(servAddr, id, name, hostname, domain string, appPort int) (*Service, error) {
	s := new(Service)
	s.AppID = id
	s.Name = name
	s.Address = hostname
	s.Domain = domain
	s.Port = appPort
	s.TTL = time.Second * 15

	client, err := consul.NewClient(newClientConfig(servAddr))
	if err != nil {
		return nil, err
	}
	s.ConsulAgent = client.Agent()

	return s, nil
}

func newClientConfig(serverAddr string) *consul.Config {
	conf := consul.DefaultConfig()
	conf.Address = serverAddr

	return conf
}

func (s *Service) GetID() string {
	return fmt.Sprintf("%s_%s", s.Name, s.AppID)
}

func (s *Service) GetFullAddr() string {
	return fmt.Sprintf("http://%s:%d/", s.Address, s.Port)
}

func (s *Service) Register() error {
	def := &consul.AgentServiceRegistration{
		ID:      s.GetID(),
		Name:    s.Name,
		Address: s.Address,
		Port:    s.Port,
		Tags:    s.getTags(),
		// Meta:    s.getMetadata(),
		Check: &consul.AgentServiceCheck{
			TTL: s.TTL.String(),
		},
	}

	if err := s.ConsulAgent.ServiceRegister(def); err != nil {
		return err
	}
	go s.UpdateTTL(def)

	return nil
}
func (s *Service) Unregister() error {
	return s.ConsulAgent.ServiceDeregister(s.GetID())
}

func (s *Service) UpdateTTL(service *consul.AgentServiceRegistration) {
	ticker := time.NewTicker(s.TTL / 2)
	for range ticker.C {
		ok, err := s.check()
		if !ok {
			if err := s.ConsulAgent.FailTTL("service:"+s.GetID(), err.Error()); err != nil {
				log.Println(err)
				fmt.Println(err)
			}
		} else {
			if err := s.ConsulAgent.PassTTL("service:"+s.GetID(), "OK"); err != nil {
				log.Println(err)
				fmt.Println(err)
			}
		}
	}
}

func (s *Service) check() (bool, error) {
	client := &http.Client{}
	healthUrl := fmt.Sprintf("%shealth", s.GetFullAddr())
	fmt.Println("HCheck: " + healthUrl)
	req, err := http.NewRequest(http.MethodGet, healthUrl, nil)
	if err != nil {
		return false, ErrServiceUnavailable
	}
	req.Header.Set("User-Agent", "Health Check")

	resp, err := client.Do(req)
	if err != nil {
		return false, ErrServiceUnavailable
	}
	defer resp.Body.Close()

	if resp.StatusCode == http.StatusOK {
		return true, nil
	}

	return false, ErrServiceUnavailable
}

func (s *Service) getMetadata() map[string]string {
	m := map[string]string{}

	return m
}

func (s *Service) getTags() []string {
	tags := []string{
		"traefik.enable=true",
		"traefik.http.routers." + s.Name + ".rule=Host(`" + s.Domain + "`)",
		"traefik.http.routers." + s.Name + ".entryPoints=https",
		"traefik.http.routers." + s.Name + ".tls=true",
		"traefik.http.routers." + s.Name + ".service=" + s.Name,
		"traefik.http.routers." + s.Name + ".middlewares=compress,requestid",
		"traefik.http.services." + s.Name + ".loadbalancer.passhostheader=false",
		"traefik.http.services." + s.Name + ".loadbalancer.server.scheme=http",
		"traefik.http.services." + s.Name + ".loadbalancer.server.port=" + strconv.Itoa(s.Port),
		"traefik.http.middlewares.compress.compress=true",
		"traefik.http.middlewares.requestid.plugin.requestid.headerName=X-Request-ID",
		// "traefik.http.services." + s.Name + ".loadbalancer.servers." + fullName + "=" + bFullAddr,
		// "traefik.http.services." + s.Name + ".loadbalancer.servers." + fullName + ".url=" + bFullAddr,
		// "traefik.http.services." + s.Name + ".loadbalancer.healthcheck.path=/health",
		// "traefik.http.services." + s.Name + ".loadbalancer.healthcheck.interval=10s",
	}

	return tags
}
